@@ -60,14 +60,45 @@ void SetExtraInfo(CommandData *Cmd,Archive &Arc,wchar *Name)
 
 
 
+bool IsRelativeSymlinkSafe(const wchar *SrcName,const wchar *TargetName)
+{
+  if (IsFullRootPath(SrcName))
+    return false;
+  int AllowedDepth=0;
+  while (*SrcName!=0)
+  {
+    if (IsPathDiv(SrcName[0]) && SrcName[1]!=0 && !IsPathDiv(SrcName[1]))
+    {
+      bool Dot=SrcName[1]=='.' && (IsPathDiv(SrcName[2]) || SrcName[2]==0);
+      bool Dot2=SrcName[1]=='.' && SrcName[2]=='.' && (IsPathDiv(SrcName[3]) || SrcName[3]==0);
+      if (!Dot && !Dot2)
+        AllowedDepth++;
+    }
+    SrcName++;
+  }
+  if (IsFullRootPath(TargetName)) // Catch root dir based /path/file paths.
+    return false;
+  for (int Pos=0;*TargetName!=0;Pos++)
+  {
+    bool Dot2=TargetName[0]=='.' && TargetName[1]=='.' && 
+              (IsPathDiv(TargetName[2]) || TargetName[2]==0) &&
+              (Pos==0 || IsPathDiv(*(TargetName-1)));
+    if (Dot2)
+      AllowedDepth--;
+    TargetName++;
+  }
+  return AllowedDepth>=0;
+}
+
+
 bool ExtractSymlink(CommandData *Cmd,ComprDataIO &DataIO,Archive &Arc,const wchar *LinkName)
 {
 #if defined(SAVE_LINKS) && defined(_UNIX)
   // For RAR 3.x archives we process links even in test mode to skip link data.
   if (Arc.Format==RARFMT15)
-    return ExtractUnixLink30(DataIO,Arc,LinkName);
+    return ExtractUnixLink30(Cmd,DataIO,Arc,LinkName);
   if (Arc.Format==RARFMT50)
-    return ExtractUnixLink50(LinkName,&Arc.FileHead);
+    return ExtractUnixLink50(Cmd,LinkName,&Arc.FileHead);
 #elif defined _WIN_ALL
   // RAR 5.0 archives store link information in file header, so there is
   // no need to additionally test it if we do not create a file.
