@@ -2,30 +2,108 @@
 static ThreadPool *GlobalPool=NULL;
 static uint GlobalPoolUseCount=0;
 
+static inline bool CriticalSectionCreate(CRITSECT_HANDLE *CritSection)
+{
+#ifdef _WIN_ALL
+  InitializeCriticalSection(CritSection);
+  return true;
+#elif defined(_UNIX)
+  return pthread_mutex_init(CritSection,NULL)==0;
+#endif
+}
+
+
+static inline void CriticalSectionDelete(CRITSECT_HANDLE *CritSection)
+{
+#ifdef _WIN_ALL
+  DeleteCriticalSection(CritSection);
+#elif defined(_UNIX)
+  pthread_mutex_destroy(CritSection);
+#endif
+}
+
+
+static inline void CriticalSectionStart(CRITSECT_HANDLE *CritSection)
+{
+#ifdef _WIN_ALL
+  EnterCriticalSection(CritSection);
+#elif defined(_UNIX)
+  pthread_mutex_lock(CritSection);
+#endif
+}
+
+
+static inline void CriticalSectionEnd(CRITSECT_HANDLE *CritSection)
+{
+#ifdef _WIN_ALL
+  LeaveCriticalSection(CritSection);
+#elif defined(_UNIX)
+  pthread_mutex_unlock(CritSection);
+#endif
+}
+
+
+static struct GlobalPoolCreateSync
+{
+  CRITSECT_HANDLE CritSection;
+  GlobalPoolCreateSync()  { CriticalSectionCreate(&CritSection); }
+  ~GlobalPoolCreateSync() { CriticalSectionDelete(&CritSection); }
+} PoolCreateSync;
+
+
 ThreadPool* CreateThreadPool()
 {
+  CriticalSectionStart(&PoolCreateSync.CritSection); 
+
   if (GlobalPoolUseCount++ == 0)
     GlobalPool=new ThreadPool(MaxPoolThreads);
+
+#ifdef RARDLL
+  // We use a simple thread pool, which does not allow to add tasks from
+  // different functions and threads in the same time. It is ok for RAR,
+  // but UnRAR.dll can be used in multithreaded environment. So if one of
+  // threads requests a copy of global pool and another copy is already
+  // in use, we create and return a new pool instead of existing global.
+  if (GlobalPoolUseCount > 1)
+  {
+    ThreadPool *Pool = new ThreadPool(MaxPoolThreads);
+    CriticalSectionEnd(&PoolCreateSync.CritSection); 
+    return Pool;
+  }
+#endif
+
+  CriticalSectionEnd(&PoolCreateSync.CritSection); 
   return GlobalPool;
 }
 
 
 void DestroyThreadPool(ThreadPool *Pool)
 {
+  CriticalSectionStart(&PoolCreateSync.CritSection); 
+
   if (Pool!=NULL && Pool==GlobalPool && GlobalPoolUseCount > 0 && --GlobalPoolUseCount == 0)
     delete GlobalPool;
+#ifdef RARDLL
+  // To correctly work in multithreaded environment UnRAR.dll creates
+  // new pools if global pool is already in use. We delete such pools here.
+  if (Pool!=NULL && Pool!=GlobalPool)
+    delete Pool;
+#endif
+
+  CriticalSectionEnd(&PoolCreateSync.CritSection); 
 }
 
 
 static THREAD_HANDLE ThreadCreate(NATIVE_THREAD_PTR Proc,void *Data)
 {
 #ifdef _UNIX
+/*
   pthread_attr_t attr;
   pthread_attr_init(&attr);
   pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
-
+*/
   pthread_t pt;
-  int Code=pthread_create(&pt,&attr,Proc,Data);
+  int Code=pthread_create(&pt,NULL/*&attr*/,Proc,Data);
   if (Code!=0)
   {
     wchar Msg[100];
@@ -59,26 +137,6 @@ static void ThreadClose(THREAD_HANDLE hThread)
 }
 
 
-static inline void CriticalSectionStart(CRITSECT_HANDLE *CritSection)
-{
-#ifdef _WIN_ALL
-  EnterCriticalSection(CritSection);
-#elif defined(_UNIX)
-  pthread_mutex_lock(CritSection);
-#endif
-}
-
-
-static inline void CriticalSectionEnd(CRITSECT_HANDLE *CritSection)
-{
-#ifdef _WIN_ALL
-  LeaveCriticalSection(CritSection);
-#elif defined(_UNIX)
-  pthread_mutex_unlock(CritSection);
-#endif
-}
-
-
 #ifdef _WIN_ALL
 static void CWaitForSingleObject(HANDLE hHandle)
 {
